"""Dashboard info tool for Optix MCP Server.

Provides an MCP tool to query dashboard connection information.
"""

from tools.dashboard_info.core import dashboard_info_impl

__all__ = ["dashboard_info_impl"]
